#include <windows.h>
#include <tchar.h>
#include <vector>
#include <string>

#pragma comment(lib, "wsock32.lib")

using namespace std;


void euc2sjis(const unsigned char *euc, unsigned char *sjis)
{
	const unsigned char ss2 = 0x8e;
	const unsigned char ss3 = 0x8f;

	while (*euc) {
		unsigned int c = *euc++;
		if (c >= 0xa1) {
			c = ((c << 8) | *euc++) & ~0x8080;
			c = _mbcjistojms(c);
			*sjis++ = c >> 8;
			*sjis++ = (unsigned char)c;
		}
		else if (c == ss2) 
			*sjis++ = *euc++;
		else if (c == ss3) {
			*sjis++ = '?';
			euc += 2;
		}
		else 
			*sjis++ = c;
	}
	*sjis = '\0';
}

void replace(std::string& str, const string& src, const string& dst)
{
	string::size_type n, nb = 0;
	
	while ((n = str.find(src,nb)) != std::string::npos)
	{
		str.replace(n,src.size(), dst);
		nb = n + dst.size();
	}	
}


void html_decode(string& script) 
{
	replace(script, "&lt;", "<");
	replace(script, "&gt;", ">");
	replace(script, "&amp;", "&");
	replace(script, "&quot;", "");
}

void parse_script(const char* html, vector<pair<int, string> >& scripts)
{
	const char* p = html;

	while (*p) {
		const char* q = strstr(p, "point=1&id=");
		if (q == NULL)
			break;

		vector<char> id;
		q += 11;
		while (*q != '&' && *q != '\0') {
			id.push_back(*q++);
		}
		id.push_back('\0');
		p = strstr(q, "<br><br><br>");
		if (p != NULL) {
			vector<char> script;
			p += 12;
			while (*p) {
				if (_ismbblead(*p)) {
					script.push_back(*p++);
				}
				else if (*p == '<')
					break;
				script.push_back(*p++);
			}
			if (!script.empty()) {
				pair<int, string> item;
				script.push_back('\0');
				item.first = atol(&id[0]);
				item.second = &script[0];
				html_decode(item.second);
				scripts.push_back(item);
			}
		}
	}
}


int url_split(const string& url, string& host, string& port, string& html)
{
	char *p, *q;
	vector<char> w;

	w.reserve(url.length() + 1);

	p = &w[0];
	strcpy(p, url.c_str());

	if (strnicmp(p, "http://", 7) != 0)
		return -1;
	p += 7;

	q = strchr(p, '/');
	if (p == NULL)
		return -1;
	html = q;
	*q = '\0';
	if (html.empty())
		html = "/";

	q = strchr(p, ':');
	if (q == NULL) 
		host = p;
	else {
		*q++ = '\0';
		port = q;
		host = p;
	}

	return 0;
}

int make_sockaddr(const string& host, const string& port, sockaddr_in& saddr)
{
	unsigned long addr;

	addr = inet_addr(host.c_str());
	if (addr == -1) {
		struct hostent *ent;
		ent = gethostbyname(host.c_str());
		if (ent == NULL)
			return -1;
		addr = *((unsigned long *)((ent->h_addr_list)[0]));
	}

	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = addr;
	saddr.sin_port = htons((unsigned short)atoi(port.c_str()));
	memset(saddr.sin_zero, 0, sizeof(saddr.sin_zero));

	return 0;
}

int wget_sub(SOCKET s, const string& url, const string& agent, string& response)
{
	int ret;
	char buf [1024];
	string host;
	string port;
	string html;

	ret = url_split(url, host, port, html);
	if (ret != 0)
		return -1;

	SOCKADDR_IN saddr;
	if (port.empty())
		port = "80";
	ret = make_sockaddr(host, port, saddr);
	if (ret != 0)
		return -1;

	ret = connect(s, (sockaddr*)&saddr, sizeof(saddr));
	if (ret == SOCKET_ERROR)
		return -1;


	memset(buf, 0, sizeof(buf));
	sprintf(buf, "GET %s HTTP/1.0\r\nHost: %s:%d\r\nUser-Agent: %s\n\n",
		html.c_str(), host.c_str(), atoi(port.c_str()), agent.c_str());
	if(send(s, buf, (int)strlen(buf), 0) == SOCKET_ERROR){
		shutdown(s, 2);
		return -1;
	}

	ret = 0;

	int len;
	while (true) {
		len = recv(s, buf, sizeof(buf) - 1, 0);
		if (len == SOCKET_ERROR) {
			ret = -1;
			break;
		}
		else if (len == 0)
			break;
		buf[len] = '\0';
		response += buf;
	}

	shutdown(s, 2);

	return ret;
}

int wget(const string& url, const string& agent, string& response)
{
	int ret;
	SOCKET s;
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s == INVALID_SOCKET)
		return -1;

	ret = wget_sub(s, url, agent, response);

	closesocket(s);

	return ret;
}

int sock_start() 
{
	int ret;
	WORD w;
	WSADATA wsaData;

	w = MAKEWORD(1,1);
	ret= WSAStartup(w, &wsaData);
	if(ret != 0)
		return -1;

	return 0;
}

int sock_end()
{
	return WSACleanup();
}


